#!/bin/bash
#
#   installer.sh
#
#   Userful Multiplier installer
#
#  	Copyright (c) 2005-2010 Userful Corporation.  All rights reserved.
#
# 	Jason Kim  <jason@userful.com>

##
## Find Linux TYPE
##
function check_os_info()
{
    if [ -e /etc/fedora-release ] || [ -e /etc/mandrake-release ] || \
       [ -e /etc/SuSE-release ] || [ -e /etc/redhat-release ]; then
       OSTYPE="rpm"

    elif [ -e /etc/xandros-deskktop-version ] || [ -e /etc/morphix_version ] || \
         [ -e /etc/freespire-release ] || [ -e /etc/linspire-release ] || \
         [ -e /etc/debian_version ]; then
       OSTYPE="deb"

   elif [ -e /etc/lsb-release ]; then
       source /etc/lsb-release
       OSTYPE=$(echo $DISTRIB_ID | tr "[A-Z" "[a-z]")

       # Check Linux Educacional
       if [ "$OSTYPE" == "linux educacional" ]; then
           OSTYPE=ubuntu
       fi

   else
       # Unsupported OS type
       # Should exit
       OSTYPE=unsupported

   fi
}
# Check system type (also in userful-bash-lib.sh)
function Check_System_Type()
{
    case $(uname -m) in
        x86_64|ia64)
            echo "lib64"
            ;;
        *)
            echo "lib"
            ;;
    esac
}
##
## Check nVidia Kernel Module and nVidia X driver
## (also in userful-bash-lib.sh)
function Check_nVidia_Driver()
{
    # Check machine type
    LIB=$(Check_System_Type)

    # Check nVidia kernel module and driver
	# Each distro has different way to install nvidia driver and use different files.
	# FC:yum, SUSE:Yast, Ubuntu:Restricted Devices Manager and Synaptic
    # TODO:
    #  - There might be a better way to check nvidia kernel module and drvier for every distros.
    #  - Each distro installs the nVidia kernel module and driver on different directory with dirrent name.
    #  - /lib is used for Kernel module instead of /lib64 in x86_64 SLED 10.1 system.
    #  - Need to check the path in other distributions.

    # Check nVidia Kernel Module
    #if [ -f /lib/modules/$(uname -r)/extra/nvidia/nvidia.ko -o  \
    #     -f /lib/modules/$(uname -r)/updates/nvidia.ko -o \
    #     -f /lib/modules/$(uname -r)/updates/dkms/nvidia.ko -o \
    #     -f /lib/modules/$(uname -r)/weak-updates/nvidia.ko -o \
    #     -f /lib/modules/$(uname -r)/kernel/drivers/video/nvidia/nvidia.ko -o \
    #     -f /lib/modules/$(uname -r)/kernel/drivers/video/nvidia.ko -o \
    #     -f /lib/modules/$(uname -r)/volatile/nvidia.ko -o \
    #     -f /var/lib/dkms/nvidia-current/kernel-$(uname -r)-i586/module/nvidia-current.ko.gz -o \
    #     -f /var/lib/dkms/nvidia/kernel-$(uname -r)-i686/module/nvidia.ko -o \
    #     -f /etc/modprobe.d/nvidia-kernel-nkc ]; then

    ##
    ## Check nVidia Kernel Module and nVidia X driver
    modprobe -q nvidia
    if lsmod | grep -q "^nvidia"; then

         # Check nVidia driver
         if [ -f /usr/$LIB/xorg/modules/drivers/nvidia_drv.o ] && \
            [ ! -L /usr/$LIB/xorg/modules/drivers/nvidia_drv.o ]; then
             echo "true" 
         elif [ -f /usr/$LIB/xorg/modules/drivers/nvidia_drv.so ] && \
              [ ! -L /usr/$LIB/xorg/modules/drivers/nvidia_drv.so ]; then
             echo "true" 
         elif [ -f /usr/$LIB/xorg/modules/drivers/nvidia_drv.so ] && \
              [ -f /usr/$LIB/nvidia-current/xorg/nvidia_drv.so ]; then
             echo "true" 
         elif [ -f /usr/$LIB/xorg/modules/updates/drivers/nvidia_drv.o ] && \
              [ ! -L /usr/$LIB/xorg/modules/updates/drivers/nvidia_drv.o ]; then
             echo "true" 
         elif [ -f /usr/$LIB/xorg/modules/updates/drivers/nvidia_drv.so ] && \
              [ ! -L /usr/$LIB/xorg/modules/updates/drivers/nvidia_drv.so ]; then
             echo "true" 
         elif [ -f /usr/$LIB/xorg/modules/drivers/nvidia-current/nvidia_drv.so ] && \
              [ ! -L /usr/$LIB/xorg/modules/drivers/nvidia-current/nvidia_drv.so ]; then
             echo "true" 
         elif [ -f /usr/X11R6/$LIB/modules/drivers/nvidia_drv.o ] && \
              [ ! -L /usr/X11R6/$LIB/modules/drivers/nvidia_drv.o ]; then
             echo "true" 
         elif [ -f /usr/X11R6/$LIB/modules/drivers/nvidia_drv.so ] && \
              [ ! -L /usr/X11R6/$LIB/modules/drivers/nvidia_drv.so ]; then
             echo "true" 
         elif [ -f /usr/X11R6/$LIB/modules/updates/drivers/nvidia_drv.o ] && \
              [ ! -L /usr/X11R6/$LIB/modules/updates/drivers/nvidia_drv.o ]; then
             echo "true" 
         elif [ -f /usr/X11R6/$LIB/modules/updates/drivers/nvidia_drv.so ] && \
              [ ! -L /usr/X11R6/$LIB/modules/updates/drivers/nvidia_drv.so ]; then
             echo "true" 
         else
             echo "false" 
         fi

    else
         # nVidia kernel module NOT exists. Return "false"
         echo "false"
    fi
}

##
## Exit from installer.sh
##
function exit_installer()
{
    #read -s -p "Please enter any key to exit from \`installer.sh'"
    exit 1
}

##
## Install UMx
##
function install_umx()
{
    pushd ./packages > /dev/null
    bash ./INSTALL
}

#-------------------
# Script start here
#-------------------

##
## Check root
if [ ! -r /proc/kmsg ]; then
    echo 
    echo "You must be root to install Userful Multiplier"
    echo
    echo "To change root user in a terminal, type \`su -' command."
    echo "If you install Userful Multiplier in graphical mode, please log on as root."
    echo
    #exit_installer
fi
    
##
## Check working directory
if [ ! -d ./packages ]; then
    echo
    echo "Please \`cd' to the newly created directory from Userful Multiplier zip file"
    echo "before you run \`installer.sh'"
    echo
    exit_installer
fi


##
## Check for Nvidia cards, if they exist, check for nvidia drivers, if no nvidia drivers, prompt to install
##
if lspci | grep -iq nvidia ; then 

	DRIVER_RESULT=$(Check_nVidia_Driver)

	if [ "$DRIVER_RESULT" == false ] ; then
		echo "You appear to have one or more NVIDIA graphic cards installed,"
		echo "but you do not have the NVIDIA driver installed. Userful multiplier"
		echo "will not work on every video output. Please see the user guide"
		echo "for more information."
		echo -n "Do you still want to install [y/N]? "
		#read ANS
        ANS=y
		if [ "$ANS" != y ] &&  [ "$ANS" != Y ] ; then
			exit_installer
		fi
	fi
fi

##
## Find Linux type.
if [ -e ./new-distro.conf ]; then

    source ./new-distro.conf
    if [ x"$PACKAGE_TYPE" == x ]; then
        echo
        echo "Please set PACKAGE_TYPE in new-distro.conf"
        exit_installer
    else
        echo
        echo "Testing Userful Multiplier on a new Linux Distro..."
        #read -s -p "Please enter any key to continue.."

        if [ -d /etc/userful ]; then
            cp -f ./new-distro.conf /etc/userful
        else
            mkdir /etc/userful
            cp -f ./new-distro.conf /etc/userful
        fi
        OSTYPE=$PACKAGE_TYPE
    fi

else
    check_os_info
fi

##  - deb packages should be available for Debian based linux
##  - Ububuntu can also use ubuntu package.
##  - rpm packages should be available for RPM based linux 
PKG_TYPE=$(cat ./packages/PKG_TYPE)

if [ "$OSTYPE" == "rpm" ]; then
    if [ "$PKG_TYPE" == "rpm" ]; then
        install_umx
    else
        echo "You have wrong version of Userful Multiplier."
        echo "Please install \`$OSTYPE' version on your system."
        exit_installer
    fi

elif [ "$OSTYPE" == "deb" ]; then
    if [ "$PKG_TYPE" == "deb" ]; then
        install_umx
    else
        echo "You have wrong version of Userful Multiplier."
        echo "Please install \`$OSTYPE' version on your system."
        exit_installer
    fi

elif [ "$OSTYPE" == "ubuntu" ]; then
    if [ "$PKG_TYPE" == "deb" ]; then
        install_umx
    elif [ "$PKG_TYPE" == "ubuntu" ]; then
        # run gdebi
        pushd ./packages > /dev/null
        gdebi *.deb
    else
        echo "You have wrong version of Userful Multiplier."
        echo "Please install \`$OSTYPE' version on your system."
        exit_installer
    fi

else
    # Unsupported OS type
    echo "Unsupported Linux system."

    echo "Please use one of the following Linux version to use Userful Multiplier."
    echo
    cat ./packages/userful-supported-distros
    exit_installer
fi

